import pygame
import time

class StateDriver(object):
    
    def __init__(self, w=1024, h=768):
        self.w = w
        self.h = h
        self.states = []
        self.alive = False
        pygame.init()
        self.screen = pygame.display.set_mode((self.w, self.h))

    @property
    def current(self):
        if self.states:
            return self.states[-1]
        return None

    @property
    def running(self):
        return self.current and self.alive

    def replace(self, state):
        self.pop()
        self.push(state)
        
    def pop(self):
        self.states.pop()

    def push(self, state):
        self.states.append(state)

    def run(self, initial_state=None):
        self.clock = pygame.time.Clock()
        
        self.alive = True
        
        if initial_state:
            self.push(initial_state)
        
        while self.running:
            state = self.current
            elapsed = self.clock.tick(60) / 1000.0
            
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    self.alive = False
                elif event.type == pygame.MOUSEBUTTONDOWN:
                    state.mouse(event.pos, event.button, True)
                elif event.type == pygame.MOUSEBUTTONUP:
                    state.mouse(event.pos, event.button, False)
            
            state.update(elapsed)
            
            self.screen.fill( (0, 0, 0) )
            state.draw(self.screen)
            pygame.display.flip()


class State(object):
    
    def __init__(self, driver=None):
        self.driver = driver
    
    def update(self, elapsed):
        pass
    
    def draw(self, screen):
        pass

    def mouse(self, pos, button, pressed):
        pass
